from pathlib import Path

def cache_path(filename):
    """cache目录"""
    dirpath = Path.home().joinpath(".cache", "weatherCN")
    if not dirpath.exists():
        dirpath.mkdir(parents=True)

    return dirpath.joinpath(filename)


def txtlen(txt):
    """返回字符串长度"""
    _len = 0
    for ch in txt:
        if ord(ch) > 0x3000:
            _len += 2
        else:
            _len += 1
    return _len


def centertxt(txt, length=12):
    """居中字符串"""
    tlen = txtlen(txt)
    rlen = (length - tlen) // 2
    llen = length - rlen - tlen
    return " " * llen + txt + " " * rlen






# Copyright (c) 2019 RedLotus <ssfdust@gmail.com>
# Author: RedLotus <ssfdust@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from pathlib import Path

from PIL import Image, ImageDraw, ImageFont

from .utils import centertxt


class Painter(object):
    """绘图模块

    根据WeatherParser获取到的信息绘制图片

    :param Image bg: 支持透明层的背景图片
    """

    def __init__(self, parser, picfinder, fontpath=""):
        self.parser = parser
        self.picfinder = picfinder
        self.fontpath = fontpath
        self.font = None
        self.bg = None
        self.img = None
        self.iconpath = Path(__file__).parent.joinpath("icons")
        self.load()

    def load(self):
        """加载内容"""
        self.set_font()
        self.load_icons()
        self.bg = self.parser.bg.convert("RGBA")
        self.img = self.bg.copy()

    def set_font(self):
        """设置字体"""
        ImageFont
        self.tifont = ImageFont.truetype(self.fontpath, 60)
        self.font = ImageFont.truetype(self.fontpath, 20)
        self.midfont = ImageFont.truetype(self.fontpath, 30)
        self.lifont = ImageFont.truetype(self.fontpath, 16)
        self.dayfont = ImageFont.truetype(self.fontpath, 13)

    def paint(self):
        """画图"""
        # ================= hover =================
        hover = Image.new("RGBA", size=self.img.size)
        arc = ImageDraw.Draw(hover)
        arc.ellipse(
            ((260, 75), (295, 110)), (177, 176, 79, int(255 * 0.6))
        )
        self.img = Image.alpha_composite(self.img, hover)
        # ================= header ================
        draw = ImageDraw.Draw(self.img)
        draw.text(
            (268, 83), self.parser.current["air_quality"], font=self.lifont
        )
        draw.text(
            (30, 40), self.parser.current["temperature"], font=self.tifont
        )
        draw.text(
            (190, 65),
            self.parser.current["cur_weather"],
            font=self.midfont,
        )
        draw.text(
            (230, 50),
            "{}更新".format(self.parser.current["updateat"]),
            font=self.lifont,
        )
        self.img.paste(
            self.picfinder.capture(self.parser.current["code"]),
            (190, 40),
            mask=self.picfinder.capture(self.parser.current["code"]),
        )
        #  ============= body right ======================
        self.img.paste(
            self.picfinder.capture(self.parser.current["dcode"], None),
            (160, 110),
            mask=self.picfinder.capture(
                self.parser.current["dcode"], None
            ),
        )
        self.img.paste(
            self.picfinder.capture(self.parser.current["ncode"], 48),
            (240, 130),
            mask=self.picfinder.capture(self.parser.current["ncode"], 48),
        )
        self.img.paste(self.up, (160, 190), mask=self.up)
        draw.text(
            (178, 186), self.parser.current["high"] + " ", font=self.lifont
        )
        self.img.paste(self.down, (230, 190), mask=self.down)
        draw.text(
            (248, 186), self.parser.current["low"] + " ", font=self.lifont
        )
        # ================ body left =================
        self.img.paste(self.wind, (35, 120), mask=self.wind)
        draw.text(
            (60, 116),
            "{} {}".format(
                self.parser.current["wind_direction"],
                self.parser.current["wind_level"],
            ),
            font=self.lifont,
        )
        self.img.paste(self.water, (35, 145), mask=self.water)
        draw.text(
            (60, 145),
            "湿度 {}".format(self.parser.current["humidity"]),
            font=self.lifont,
        )
        draw.text(
            (60, 181), self.parser.current["weather"], font=self.lifont
        )
        # ============ below ========================
        draw.line((20, 216, 316, 216), fill=(122, 109, 109, 250))
        draw.line((118, 216, 118, 366), fill=(122, 109, 109, 250))
        draw.line((216, 216, 216, 366), fill=(122, 109, 109, 250))
        # ============ day one ======================

        if len(self.parser.forecast) >= 1:
            draw.text(
                (33, 226),
                centertxt(self.parser.forecast[0]["date"]),
                font=self.dayfont,
            )
            draw.text(
                (33, 244),
                centertxt(self.parser.forecast[0]["weekday"]),
                font=self.dayfont,
            )
            self.img.paste(
                self.picfinder.capture(self.parser.forecast[0]["dcode"]),
                (50, 265),
                mask=self.picfinder.capture(self.parser.forecast[0]["dcode"]),
            )
            draw.text(
                (33, 300),
                centertxt(format(self.parser.forecast[0]["weather"])),
                font=self.dayfont,
            )
            draw.text(
                (33, 320),
                centertxt(
                    "{} ~ {}".format(
                        self.parser.forecast[0]["high"],
                        self.parser.forecast[0]["low"],
                    )
                ),
                font=self.dayfont,
            )

        # ================= day two ===================================
        if len(self.parser.forecast) >= 2:
            draw.text(
                (33 + 100, 226),
                centertxt(self.parser.forecast[1]["date"]),
                font=self.dayfont,
            )
            draw.text(
                (33 + 100, 244),
                centertxt(self.parser.forecast[1]["weekday"]),
                font=self.dayfont,
            )
            self.img.paste(
                self.picfinder.capture(self.parser.forecast[1]["dcode"]),
                (50 + 100, 265),
                mask=self.picfinder.capture(self.parser.forecast[1]["dcode"]),
            )
            draw.text(
                (33 + 100, 300),
                centertxt(format(self.parser.forecast[1]["weather"])),
                font=self.dayfont,
            )
            draw.text(
                (33 + 100, 320),
                centertxt(
                    "{} ~ {}".format(
                        self.parser.forecast[1]["high"],
                        self.parser.forecast[1]["low"],
                    )
                ),
                font=self.dayfont,
            )

        # ================== day three ==========================
        if len(self.parser.forecast) >= 3:
            last_index = 2
            draw.text(
                (33 + 200, 226),
                centertxt(self.parser.forecast[last_index]["date"]),
                font=self.dayfont,
            )
            draw.text(
                (33 + 200, 244),
                centertxt(self.parser.forecast[last_index]["weekday"]),
                font=self.dayfont,
            )
            self.img.paste(
                self.picfinder.capture(self.parser.forecast[last_index]["dcode"]),
                (50 + 200, 265),
                mask=self.picfinder.capture(self.parser.forecast[last_index]["dcode"]),
            )
            draw.text(
                (33 + 200, 300),
                centertxt(format(self.parser.forecast[last_index]["weather"])),
                font=self.dayfont,
            )
            draw.text(
                (33 + 200, 320),
                centertxt(
                    "{} ~ {}".format(
                        self.parser.forecast[last_index]["high"],
                        self.parser.forecast[last_index]["low"],
                    )
                ),
                font=self.dayfont,
            )

    def load_icons(self):
        """加载图标"""
        self.up = Image.open(self.iconpath.joinpath("up.png")).convert(
            "RGBA"
        )
        self.down = Image.open(self.iconpath.joinpath("down.png")).convert(
            "RGBA"
        )
        self.water = Image.open(
            self.iconpath.joinpath("water.png")
        ).convert("RGBA")
        self.wind = Image.open(self.iconpath.joinpath("wind.png")).convert(
            "RGBA"
        )
        self.pm = Image.open(self.iconpath.joinpath("pm.png")).convert(
            "RGBA"
        )

    def crop(self):
        return self.img.crop((21, 28, 315, 358))


def paint_icon(parser, picfinder, code=""):
    parser = parser
    picfinder = picfinder
    bg = Image.new("RGBA", size=(32, 32), color="#ffffff00")
    img = bg.copy()
    hover = Image.new("RGBA", size=img.size)
    img = Image.alpha_composite(img, hover)
    img.paste(
        picfinder.capture(parser.current["code"]),
        (0, 0),
        mask=picfinder.capture(parser.current["code"]),
    )
    return img



# Copyright (c) 2019 RedLotus <ssfdust@gmail.com>
# Author: RedLotus <ssfdust@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""获取天气预报

"""

import arrow
import requests
import json
import regex

class WeatherRepoter(object):
    """
    获取天气预报

    :param int year: 年份
    :param int month: 月份
    :param str location: 地区

    ```location``` 地区参数可以为整形也可以为字符串
    """

    def __init__(self, year, month, location):
        self.url = self.urlmaker(year, month, location)
        self.cur_url = self.cururlmaker(location)
        self.headers = self.header_maker(location)
        self.calendar = None  # 日历字典
        self.current = None  # 当前天气字典
        self.forecast = None  # 自今日起后4日日历字典
        self.get_data()
        self.filter_calendar()

    def get_data(self):
        """
        获取日历数据以及实时数据

        访问地址时，需要带上时间参数以及Referer的headers
        """
        # 生成时间参数
        params = {'_': arrow.get().timestamp() * 1000}
        # 获取日历数据
        resp = requests.get(self.url, headers=self.headers, params=params)
        self.calendar = self.parse(resp.content)
        # 获取实时数据
        resp = requests.get(self.cur_url, headers=self.headers, params=params)
        self.current = self.parse(resp.content)

    def cururlmaker(self, location):
        """当前天气URL"""
        return f"http://d1.weather.com.cn/sk_2d/{location}.html"

    def filter_calendar(self):
        """过滤最近四天的数据"""
        forecast = list(filter(lambda x: True if x["date"] in self.datelst else False, self.calendar))
        # 重新排序
        forecast.sort(key=lambda x: x["date"])
        self.forecast = forecast

    @property
    def datelst(self):
        """最近四天日期"""
        now = arrow.now()
        return [now.shift(days=i).format("YYYYMMDD") for i in range(0, 4)]

    def parse(self, data):
        """提取json字符串，并转为dict"""
        data = data.decode("utf-8")
        jsonstr = regex.search(r"[\s=]([{\[].*?[}\]])$", data).group(1)
        return json.loads(jsonstr)

    def header_maker(self, location):
        """生成合法headers"""
        return {"Referer": f"http://www.weather.com.cn/weather40dn/{location}.shtml",
                "User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:66.0) Gecko/20100101 Firefox/66.0"}

    def urlmaker(self, year, month, location):
        """日历天气URL"""
        return f"http://d1.weather.com.cn/calendarFromMon/{year}/{location}_{year}{month:02}.html"



import requests
import json
from time import time


def get_code_from_name(name: str) -> str:
    url = "http://toy1.weather.com.cn/search?cityname={city}&callback=&_={time}".format(city=name, time=int(time() * 1000))
    headers = {
        "Sec-Fetch-Dest": "script",
        "Sec-Fetch-Mode": "no-cors",
        "Sec-Fetch-Site": "same-site",
        "Sec-GPC": "1",
        "Referer": "https://www.weather.com.cn/",
    }

    req = requests.get(url, cookies={"f_city": "beijing"}, headers=headers)
    results = json.loads(req.text[1:-1])
    for r in results:
        return r['ref'].split("~")[0]

def get_code(name: str) -> str:
    if name.isnumeric():
        return name
    return get_code_from_name(name)



"""Font module

Select the best match font"""
from platform import system
from pathlib import Path
from typing import Optional

class FontSelector:
    """The font selection class

    The class try to match the best font via the following actions:
    1. check whether the font given is a path
    2. check the system font:
        1. if user's system is Liunx, try to get it from fclist
        2. if user's system is Windows, try to get it from system
    3. use one fallback font:
        1. fcmatch lang=zh-cn
        2. msyh.ttf or sumttf
    """
    def __init__(self, font: str = ""):
        self.desired_font = font
        self.best_font_path: Optional[Path] = None
        self.usefc = "linux" in system().lower()

    def _is_font_a_path(self):
        """check if it's a font path"""
        if self.best_font_path:
            return
        maybe_font_path = Path(self.desired_font)
        if maybe_font_path.exists() and maybe_font_path.is_file():
            self.best_font_path = maybe_font_path

    def __load_from_fc(self):
        from fclist import fclist
        for f in fclist(family=self.desired_font):
            self.best_font_path = Path(f.file)
            break

    def __load_from_windows(self):
        for fontpath in Path("C:\Windows\fonts").iterdir():
            if fontpath.is_file() and fontpath.name == self.desired_font:
                self.best_font_path = fontpath
                break

    def __load_fc_fallback(self):
        from fclist import fcmatch
        self.best_font_path = Path(fcmatch("sans-serif:lang=zh-cn").file)

    def __load_windows_fallback(self):
        for name in ["msyh", "simsun"]:
            for fontpath in Path("C:\Windows\fonts").iterdir():
                if name in fontpath.name.lower():
                    self.best_font_path = fontpath
                    break
            if self.best_font_path:
                break

    def _is_font_a_system_font_name(self):
        """check weather it is a system font name"""
        if self.best_font_path:
            return
        if self.usefc:
            self.__load_from_fc()
        else:
            self.__load_from_windows()

    def _is_font_no_match(self):
        """use the fallback font"""
        if self.best_font_path:
            return
        if self.usefc:
            self.__load_fc_fallback()
        else:
            self.__load_window_fallback()

    def select_the_best(self) -> Path:
        self._is_font_a_path()
        self._is_font_a_system_font_name()
        self._is_font_no_match()
        return self.best_font_path



# Copyright (c) 2019 RedLotus <ssfdust@gmail.com>
# Author: RedLotus <ssfdust@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import regex
import io
import requests
from PIL import Image
import css_parser

class PicFinder(object):
    """获取css，提取背景

    天气图片是一张图片，通过background-postion来显示
    通过PIL剪切图片，来获取单个的天气图片

    :param str cssulr: css地址
    :param str imgurl: 图片地址
    :param regex.Regex reg: 匹配正则
    ```reg``` 匹配 {"d00".. "d10", "n00" .. "n10"}
              等天气style d开头为白天 n为夜晚
    :param int cropw: 图片宽度
    :param int croph: 图片高度
    :param dict mapping: 类别映射
    """

    def __init__(self):
        self.cssulr = "http://i.tq121.com.cn/c/weather2017/headStyle_1.css"
        self.imgurl = "https://i.tq121.com.cn/i/weather2017/weather_icon_d40l_b.png"
        self.reg = regex.compile(r"\.[nd]\d+$")
        self.cropw = 70
        self.croph = 77
        self.mapping = {}
        self.download()
        self.get_css()

    def download(self):
        """下载天气图片"""
        resp = requests.get(self.imgurl)
        picbin = io.BytesIO(resp.content)
        self.img = Image.open(picbin)

    def capture(self, kls, size=32):
        """根据类别剪切，默认大小为32x32, None时为默认大小"""
        box = self.mapping[kls]
        crop = self.img.crop(box)
        return crop.resize((size, size)) if size else crop

    def get_box(self, pos):
        """根据偏移量，获取PIL剪切矩阵"""
        width, height = pos.split()
        width = -int(width.replace("px", ""))
        height = -int(height.replace("px", ""))
        return (width, height, width + self.cropw, height + self.croph)

    def get_css(self):
        """获取css 并转换为mapping"""
        # 下载css
        sheet = css_parser.parseUrl(self.cssulr, validate=False, media=False)
        for rule in sheet.cssRules:
            if hasattr(rule, "selectorList"):
                for selector in rule.selectorList:
                    if self.reg.search(selector.selectorText):
                        sel = selector.selectorText.split('.')[1]
                        self.mapping[sel] = self.get_box(rule.style["background-position"])



from weathercn.font import FontSelector
from pathlib import Path
import pytest

class TestFontSelector:
    def test_is_font_a_path(self):
        path = "/home/ssfdust/NextCloud/备份/Fonts/FZLanTYK_Cu.ttf"
        selector = FontSelector(path)
        assert selector.select_the_best() == Path(path)

    @pytest.mark.parametrize("name", ["WenQuanYi Zen Hei","文泉驛正黑","文泉驿正黑"])
    def test_is_system_font(self, name: str):
        selector = FontSelector(name)
        path = "/usr/share/fonts/wenquanyi/wqy-zenhei/wqy-zenhei.ttc"
        assert selector.select_the_best() == Path(path)

    def test_system_font_fallback(self):
        selector = FontSelector()
        path = "/usr/share/fonts/adobe-source-sans/SourceSansPro-Regular.otf"
        assert selector.select_the_best() == Path(path)




#!/bin/env python3
# Copyright (c) 2019 RedLotus <ssfdust@gmail.com>
# Author: RedLotus <ssfdust@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from argparse import ArgumentParser
from datetime import datetime
from weathercn.citycode import get_code
from weathercn.font import FontSelector

from weathercn.paint import Painter, paint_icon
from weathercn.parser import WeatherParser
from weathercn.picfinder import PicFinder
from weathercn.reporter import WeatherRepoter
from weathercn.utils import cache_path


def main():
    parser = ArgumentParser()
    parser.add_argument("location", help="地址")
    parser.add_argument("-f", "--fontpath", help="字体路径", default="")
    args = parser.parse_args()

    p = PicFinder()
    time = datetime.now()
    wr = WeatherRepoter(time.year, time.month, get_code(args.location))
    par = WeatherParser(wr.current, wr.forecast)
    par.parse()
    par.to_json(cache_path("weather.json"))
    selector = FontSelector(args.fontpath)
    fontpath = str(selector.select_the_best())
    pa = Painter(par, p, fontpath)
    pa.load()
    pa.paint()
    bg = pa.crop()
    bg.save(cache_path("weather.png"))

if __name__ == '__main__':
    main()



# Copyright (c) 2019 RedLotus <ssfdust@gmail.com>
# Author: RedLotus <ssfdust@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import io

import arrow
import json
import requests
from PIL import Image

class WeatherParser(object):

    def __init__(self, current, forecast):
        self._current = current
        self.current = None
        self._forecast = forecast
        self.forecast = None
        self.sz = ["初一", "初八", "十四",
                   "十五", "十八", "廿三",
                   "廿四", "廿八", "廿九", "三十"]
        self.bg = None

    def parse(self):
        """进行处理"""
        self.parse_current()
        self.get_background()
        self.parse_forecast()

    def to_json(self, fpath=None):
        data = {
            "current": self.current,
            "forcast": self.forecast
        }
        if fpath:
            with open(fpath, 'w') as f:
                json.dump(data, f, indent=4, ensure_ascii=False)
        else:
            return json.dumps(data, indent=4, ensure_ascii=False)

    def to_cnday(self, date):
        """转换日期为星期"""
        return arrow.get(date, "YYYYMMDD").format("dddd", "zh_CN")

    def to_cndate(self, date):
        """转换为月日"""
        return arrow.get(date, "YYYYMMDD").format("MMMMD日", "zh_CN")

    def parse_forecast(self):
        """处理未来三天天气预报内容"""
        self.forecast = [
            {
                "date": self.to_cndate(item["date"]),
                "weekday": self.to_cnday(item["date"]),
                "high": item["max"],
                "low": item["min"],
                "dcode": "d{}".format(item["c1"]),
                "ncode": "n{}".format(item["c2"]),
                "weather": item["w1"]
            } for item in self._forecast[1:]
        ]

    def parse_current(self):
        """处理当前天气内容"""
        self.current = {
            "humidity": self._current["SD"],
            "wind_direction": self._current["WD"],
            "wind_level": self._current["WS"],
            "air_quality": self._current["aqi_pm25"],
            "air_pressure": self._current["qy"],
            "updateat": self._current["time"],
            "cur_weather": self._current['weather'],
            "temperature": "%s ℃" % self._current["temp"],
            "weather": self._forecast[0]["w1"],
            "dcode": "d{}".format(self._forecast[0]["c1"]),
            "ncode": "d{}".format(self._forecast[0]["c2"]),
            "high": "%s ℃" % self._forecast[0]["max"],
            "low": "%s ℃" % self._forecast[0]["min"],
            "code": self._current["weathercode"],
            "unsuited": self._forecast[0]["alins"],
            "suited": self._forecast[0]["als"],
            "lunar": "{}{}".format(self._forecast[0]["nlyf"],
                                   self._forecast[0]["nl"]),
            "shizhai": "十斋日" if self._forecast[0]["nl"] in self.sz else ""
        }

    def get_background(self):
        """获取背景"""
        code = self.current["code"][1:]
        bg = "weatherBg01" if code not in self.mapping else self.mapping[code]

        url = "https://i.tq121.com.cn/i/weather2017/{}.jpg".format(bg)
        resp = requests.get(url)
        self.bg = Image.open(io.BytesIO(resp.content))

    @property
    def mapping(self):
        """编码背景映射"""
        return {
            "01": "weatherBg02",
            "02": "weatherBg03",
            "03": "weatherBg04",
            "06": "weatherBg04",
            "07": "weatherBg04",
            "08": "weatherBg04",
            "09": "weatherBg04",
            "10": "weatherBg04",
            "11": "weatherBg04",
            "12": "weatherBg04",
            "19": "weatherBg04",
            "21": "weatherBg04",
            "22": "weatherBg04",
            "23": "weatherBg04",
            "24": "weatherBg04",
            "25": "weatherBg04",
            "301": "weatherBg04",
            "97": "weatherBg04",
            "04": "weatherBg05",
            "05": "weatherBg05",
            "13": "weatherBg06",
            "14": "weatherBg06",
            "15": "weatherBg06",
            "16": "weatherBg06",
            "17": "weatherBg06",
            "26": "weatherBg06",
            "27": "weatherBg06",
            "28": "weatherBg06",
            "302": "weatherBg06",
            "98": "weatherBg06",
            "18": "weatherBg07",
            "32": "weatherBg07",
            "49": "weatherBg07",
            "57": "weatherBg07",
            "58": "weatherBg07",
            "20": "weatherBg08",
            "31": "weatherBg08",
            "29": "weatherBg09",
            "30": "weatherBg09",
            "53": "weatherBg10",
            "54": "weatherBg10",
            "55": "weatherBg10",
            "56": "weatherBg10"
        }
